/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2023 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.api.client.model.loading.v1;

import net.minecraft.class_1087;
import net.minecraft.class_1091;
import net.minecraft.class_1092;
import net.minecraft.class_2960;
import org.jetbrains.annotations.Nullable;

/**
 * Fabric-provided helper methods for {@link class_1092}.
 *
 * <p>Note: This interface is automatically implemented on the {@link class_1092} via Mixin and interface injection.
 */
public interface FabricBakedModelManager {
	/**
	 * An alternative to {@link class_1092#method_4742(class_1091)} that accepts an
	 * {@link class_2960} instead. Models loaded using {@link ModelLoadingPlugin.Context#addModels}
	 * do not have a corresponding {@link class_1091}, so the vanilla method cannot be used to
	 * retrieve them. The {@link class_2960} that was used to load them can be used in this method
	 * to retrieve them.
	 *
	 * <p><b>This method, as well as its vanilla counterpart, should only be used after the
	 * {@link class_1092} has completed reloading.</b> Otherwise, the result will be
	 * outdated or null.
	 *
	 * @param id the id of the model
	 * @return the model
	 */
	@Nullable
	default class_1087 getModel(class_2960 id) {
		throw new UnsupportedOperationException("Implemented via mixin.");
	}
}
